package org.jvnet.hifaces.binding;

import java.io.Serializable;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.PropertyNotFoundException;

/**
 * Variable value binding.
 * 
 * @author valikov
 * 
 * @param <T>
 *            type of variable.
 * @param <V>
 *            type of value.
 */
public class VariableValueBinding<T, V> extends ValueBinding<V> implements
		Serializable {

	private static final long serialVersionUID = 1L;

	private final String variableName;

	public VariableValueBinding(final String var) {
		this.variableName = var;
	}

	public boolean isReadOnly(FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		return true;
	}

	@SuppressWarnings("unchecked")
	public void setValue(FacesContext facesContext, Object value)
			throws EvaluationException, PropertyNotFoundException {
		final T variable = (T) facesContext.getApplication()
				.getVariableResolver().resolveVariable(facesContext,
						variableName);
		final V newValue = (V) value;
		set(variable, value);

	}

	@SuppressWarnings("unchecked")
	public V getValue(FacesContext facesContext) throws EvaluationException,
			PropertyNotFoundException {

		final T value = (T) facesContext.getApplication().getVariableResolver()
				.resolveVariable(facesContext, variableName);
		return get(facesContext, value);
	}

	public V get(FacesContext facesContext, T var) {
		return get(var);
	}

	@SuppressWarnings("unchecked")
	public V get(T var) {
		return (V) var;
	}

	public void set(Object variable, Object value) {
		throw new UnsupportedOperationException();
	}

	public Class getType(FacesContext facesContext) throws EvaluationException,
			PropertyNotFoundException {
		return Object.class;
	}

	@Override
	public String getExpressionString() {
		return "#{" + variableName + "}";
	}
}
