package org.jvnet.hifaces.binding.message;

import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.PropertyNotFoundException;

import org.jvnet.hifaces.binding.ValueBinding;
import org.springframework.context.MessageSource;
import org.springframework.web.jsf.FacesContextUtils;

public abstract class AbstractMessageValueBinding extends ValueBinding {

	private static final long serialVersionUID = 1L;

	private transient MessageSource messageSource;

	protected MessageSource getMessageSource(final FacesContext facesContext) {
		if (messageSource == null) {
			messageSource = FacesContextUtils
					.getWebApplicationContext(facesContext);
		}
		return messageSource;
	}

	public Object getValue(final FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		final MessageSource wac = getMessageSource(facesContext);
		return getMessage(facesContext, wac);
	}

	protected abstract String getMessage(final FacesContext facesContext,
			final MessageSource messageSource);

	protected String getMessage(final FacesContext facesContext,
			final MessageSource messageSource, final String messageId,
			final Object[] arguments) {
		final Locale locale = getLocale(facesContext);
		final String message;
		if (messageSource != null) {
			message = messageSource.getMessage(messageId, arguments, "???"
					+ messageId + "???", locale);
		} else {
			message = "???" + messageId + "???";
		}
		return message;
	}

	protected Locale getLocale(final FacesContext facesContext) {
		return facesContext.getViewRoot().getLocale();
	}

	public void setValue(final FacesContext facesContext, final Object o)
			throws EvaluationException, PropertyNotFoundException {
	}

	public boolean isReadOnly(final FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		return true;
	}

	public Class getType(final FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		return String.class;
	}

}
