package org.jvnet.hifaces.convert;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;


public class StringCollectionConverter implements Converter {

	private static final String CONVERSION_MESSAGE_ID = "org.jvnet.hifaces.convert.StringCollectionConverter.CONVERSION";

	public static final String CONVERTER_ID = "org.jvnet.hifaces.StringCollection";

	private final Converter converter;

	public Converter getConverter() {
		return converter;
	}

	public StringCollectionConverter(final Converter converter) {
		this.converter = converter;
	}

	public String getAsString(FacesContext context, UIComponent component,
			Object value) throws ConverterException {

		if (context == null)
			throw new NullPointerException("facesContext");
		if (component == null)
			throw new NullPointerException("uiComponent");

		if (value == null) {
			return "";
		} else {
			final StringBuffer sb = new StringBuffer();
			final List list = (List) value;
			for (Iterator iterator = list.iterator(); iterator.hasNext();) {
				final Object object = (Object) iterator.next();
				try {
					final String string = getConverter().getAsString(context,
							component, object);
					sb.append(string);
				} catch (Exception ex) {
					throw new ConverterException("Cannot convert value.", ex);
				}
				if (iterator.hasNext()) {
					sb.append("\n");
				}
			}
			return sb.toString();
		}
	}

	public Object getAsObject(FacesContext context, UIComponent component,
			String value) throws ConverterException {
		if (value == null) {
			return null;
		} else {
			final List result = new ArrayList();
			final StringTokenizer st = new StringTokenizer(value, "\n");
			int line = 0;
			while (st.hasMoreTokens()) {
				try {
					final String draftToken = st.nextToken();
					line = line + 1;
					final String token = draftToken.trim();
					if (token.length() > 0) {
						final Object object = getConverter().getAsObject(
								context, component, token);
						result.add(object);
					}
				} catch (Exception ex) {

					throw new ConverterException(MessageUtils.getErrorMessage(
							context, CONVERSION_MESSAGE_ID, new Object[] {
									value, component.getId(),
									Integer.valueOf(line) }), ex);
				}
			}
			return result;
		}
	}
}
