package org.jvnet.hifaces.event;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class ListDataListenerList extends AbstractWeakListenerList {

  public void addListDataListener(ListDataListener listener) {
    add(listener);
  }

  public void removeListDataListener(ListDataListener listener) {
    remove(listener);
  }

  public void clearListDataListeners() {
    clear();
  }

  public void fireListDataEvent(ListDataEvent event) {
    fire(event);
  }

  /**
   * Fires interval added event.
   *
   * @param source    source object - list that is changed.
   * @param fromIndex lower index.
   * @param toIndex   upper index.
   */
  public void fireIntervalAddedEvent(final Object source, final int fromIndex, final int toIndex) {
    fireListDataEvent(new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, fromIndex, toIndex));
  }

  /**
   * Fires interval removed event.
   *
   * @param source    source object.
   * @param fromIndex lower index.
   * @param toIndex   upper index.
   */
  public void fireIntervalRemovedEvent(final Object source, final int fromIndex, final int toIndex) {
    fireListDataEvent(new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, fromIndex, toIndex));
  }

  /**
   * Fires contents changed event.
   *
   * @param source    source object.
   * @param fromIndex lower index.
   * @param toIndex   upper index.
   */
  public void fireContentsChangedEvent(final Object source, final int fromIndex, final int toIndex) {
    fireListDataEvent(new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, fromIndex, toIndex));
  }

  /**
   * Fires entry added event.
   *
   * @param source source object.
   * @param index  position of the added entry.
   */
  public void fireEntryAddedEvent(final Object source, final int index) {
    fireIntervalAddedEvent(source, index, index);
  }

  /**
   * Fires entry removed event.
   *
   * @param source source object.
   * @param index  position of the added entry.
   */
  public void fireEntryRemovedEvent(final Object source, final int index) {
    fireIntervalRemovedEvent(source, index, index);
  }

  /**
   * Fires entry changed event.
   *
   * @param source source object.
   * @param index  position of the added entry.
   */
  public void fireEntryChangedEvent(final Object source, final int index) {
    fireContentsChangedEvent(source, index, index);
  }

  protected void fire(Object listenerDraft, Object eventDraft) {
    final ListDataListener listener = (ListDataListener) listenerDraft;
    final ListDataEvent event = (ListDataEvent) eventDraft;
    switch (event.getType()) {
      case ListDataEvent.INTERVAL_ADDED:
        listener.intervalAdded(event);
        break;
      case ListDataEvent.INTERVAL_REMOVED:
        listener.intervalRemoved(event);
        break;
      case ListDataEvent.CONTENTS_CHANGED:
        listener.contentsChanged(event);
        break;
    }
  }

}
