package org.jvnet.hifaces.springframework.factory.config;

import org.jvnet.hifaces.io.util.TransientValue;
import org.jvnet.hifaces.servlet.http.RequestTrackingFilter;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

public class TransitionScope implements Scope {

	public Object get(String name, ObjectFactory objectFactory) {

		final RequestAttributes attributes = RequestContextHolder
				.currentRequestAttributes();

		// Check request scope
		{
			final Object object = attributes.getAttribute(name,
					RequestAttributes.SCOPE_REQUEST);
			if (object != null) {
				return object;
			}
		}

		// Check session for the bean from the previous request
		final Long previousRequestId = (Long) attributes
				.getAttribute(
						RequestTrackingFilter.PREVIOUS_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
						RequestAttributes.SCOPE_REQUEST);

		if (previousRequestId != null) {
			final String key = getKey(name, previousRequestId);
			final Object attribute = attributes.getAttribute(key,
					RequestAttributes.SCOPE_SESSION);
			if (attribute != null) {
				attributes
						.removeAttribute(key, RequestAttributes.SCOPE_SESSION);
				final TransientValue<Object> value = (TransientValue<Object>) attribute;
				final Object object = value.get();
				if (object != null) {
					attributes.setAttribute(name, object,
							RequestAttributes.SCOPE_REQUEST);
					return object;
				}
			}
		}

		final Long currentRequestId = (Long) attributes
				.getAttribute(
						RequestTrackingFilter.CURRENT_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
						RequestAttributes.SCOPE_REQUEST);

		if (currentRequestId != null) {
			final Object object = objectFactory.getObject();
			final String key = getKey(name, currentRequestId);
			final TransientValue<Object> attribute = new TransientValue<Object>(object);
			attributes.setAttribute(key, attribute,
					RequestAttributes.SCOPE_SESSION);
			attributes.setAttribute(name, object,
					RequestAttributes.SCOPE_REQUEST);
			return object;
		}

		{
			final Object object = objectFactory.getObject();
			attributes.setAttribute(name, object,
					RequestAttributes.SCOPE_REQUEST);
			return object;
		}
	}

	public Object remove(String name) {

		final RequestAttributes attributes = RequestContextHolder
				.currentRequestAttributes();

		// Check session for the bean from the previous request
		final Long previousRequestId = (Long) attributes
				.getAttribute(
						RequestTrackingFilter.PREVIOUS_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
						RequestAttributes.SCOPE_REQUEST);

		final Long currentRequestId = (Long) attributes
				.getAttribute(
						RequestTrackingFilter.CURRENT_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
						RequestAttributes.SCOPE_REQUEST);

		// Check request scope
		{
			final Object object = attributes.getAttribute(name,
					RequestAttributes.SCOPE_REQUEST);
			if (object != null) {
				if (previousRequestId != null) {
					attributes.removeAttribute(getKey(name, previousRequestId),
							RequestAttributes.SCOPE_SESSION);
				}

				if (currentRequestId != null) {
					attributes.removeAttribute(getKey(name, currentRequestId),
							RequestAttributes.SCOPE_SESSION);
				}
				return object;
			}
		}

		if (previousRequestId != null) {
			final String key = getKey(name, previousRequestId);
			final Object attribute = attributes.getAttribute(key,
					RequestAttributes.SCOPE_SESSION);
			if (attribute != null) {
				attributes
						.removeAttribute(key, RequestAttributes.SCOPE_SESSION);
				final TransientValue<Object> reference = (TransientValue<Object>) attribute;
				final Object object = reference.get();

				if (currentRequestId != null) {
					attributes.removeAttribute(getKey(name, currentRequestId),
							RequestAttributes.SCOPE_SESSION);
				}

				return object;
			}
		}

		if (currentRequestId != null) {
			attributes.removeAttribute(getKey(name, currentRequestId),
					RequestAttributes.SCOPE_SESSION);
			return null;
		}

		return null;
	}

	protected String getKey(String name, final Long id) {
		return name + "<" + id + ">";
	}
	
	public String getConversationId() {
		return null;
	}
	
	public void registerDestructionCallback(String name, Runnable callback) {
	}
}
