package org.jvnet.hifaces.springframework.factory.config;

import javax.faces.context.FacesContext;

import org.jvnet.hifaces.io.util.TransientValue;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

public class ViewScope implements Scope {

	@SuppressWarnings("unchecked")
	public Object get(String name, ObjectFactory objectFactory) {

		final RequestAttributes attributes = RequestContextHolder
				.currentRequestAttributes();

		// Check request scope
		{
			final Object object = attributes.getAttribute(name,
					RequestAttributes.SCOPE_REQUEST);
			if (object != null) {
				return object;
			}
		}

		{

			final String viewId = getViewId();

			final String key = getKey(name, viewId);

			final Object attribute = attributes.getAttribute(key,
					RequestAttributes.SCOPE_SESSION);

			if (attribute != null) {
				final TransientValue<Object> value = (TransientValue<Object>) attribute;
				final Object object = value.get();
				if (object != null) {
					attributes.setAttribute(name, object,
							RequestAttributes.SCOPE_REQUEST);
					return object;
				}
			}
			{
				final Object object = objectFactory.getObject();
				final TransientValue<Object> newAttribute = new TransientValue<Object>(
						object);
				attributes.setAttribute(key, newAttribute,
						RequestAttributes.SCOPE_SESSION);
				attributes.setAttribute(name, object,
						RequestAttributes.SCOPE_REQUEST);
				return object;
			}
		}
	}

	protected String getViewId() {
		final FacesContext facesContext = FacesContext.getCurrentInstance();
		final String viewId = facesContext.getViewRoot().getViewId();
		return viewId;
	}

	public Object remove(String name) {

		final RequestAttributes attributes = RequestContextHolder
				.currentRequestAttributes();

		final String viewId = getViewId();

		final String key = getKey(name, viewId);

		// Check request scope
		{
			final Object object = attributes.getAttribute(name,
					RequestAttributes.SCOPE_REQUEST);
			if (object != null) {
				attributes
						.removeAttribute(key, RequestAttributes.SCOPE_SESSION);
				return object;
			}
		}

		final Object attribute = attributes.getAttribute(key,
				RequestAttributes.SCOPE_SESSION);
		if (attribute != null) {
			attributes.removeAttribute(key, RequestAttributes.SCOPE_SESSION);
			final TransientValue<Object> reference = (TransientValue<Object>) attribute;
			final Object object = reference.get();
			return object;
		}

		return null;
	}

	protected String getKey(String name, String id) {
		return name + "<" + id + ">";
	}
	
	public String getConversationId() {
		return null;
	}
	
	public void registerDestructionCallback(String name, Runnable callback) {
	}

}
